/*
 * talkatu
 * Copyright (C) 2017-2018 Gary Kramlich <grim@reaperworld.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>

#include <talkatu/talkatu.h>

#include "talkatudemomessage.h"

#define TALKATU_TYPE_DEMO            (talkatu_demo_get_type())
#define TALKATU_DEMO(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), TALKATU_TYPE_DEMO, TalkatuDemo))
#define TALKATU_DEMO_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), TALKATU_TYPE_DEMO, TalkatuDemoClass))
#define TALKATU_IS_DEMO(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), TALKATU_TYPE_DEMO))
#define TALKATU_IS_DEMO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TALKATU_TYPE_DEMO))
#define TALKATU_DEMO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), TALKATU_TYPE_DEMO, TalkatuDemoClass))

typedef struct {
	GtkWindow parent;

	GtkWidget *view_history;
	GtkWidget *editor;

	GtkRadioToolButton *toggle_plain;
	GtkRadioToolButton *toggle_whole;
	GtkRadioToolButton *toggle_html;
	GtkRadioToolButton *toggle_markdown;

	GtkTextBuffer *buffer_plain;
	GtkTextBuffer *buffer_whole;
	GtkTextBuffer *buffer_html;
	GtkTextBuffer *buffer_markdown;
	GtkTextBuffer *buffer_history;
} TalkatuDemo;

typedef struct {
	GtkWindowClass parent;
} TalkatuDemoClass;

G_DEFINE_TYPE(TalkatuDemo, talkatu_demo, GTK_TYPE_WINDOW);

static gboolean exit_early = FALSE;

static void
talkatu_demo_insert_html_cb(GtkButton *toggle, gpointer data) {
	TalkatuDemo *demo = (TalkatuDemo *)data;
	GtkWidget *dialog = NULL;
	GtkFileFilter *filter = NULL;
	gint res;

	dialog = gtk_file_chooser_dialog_new(
		_("insert html..."),
		GTK_WINDOW(demo),
		GTK_FILE_CHOOSER_ACTION_OPEN,
		"OK", GTK_RESPONSE_ACCEPT,
		"Cancel", GTK_RESPONSE_REJECT,
		NULL
	);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.html");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);

	res = gtk_dialog_run(GTK_DIALOG(dialog));
	if(res == GTK_RESPONSE_ACCEPT) {
		gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gchar *contents = NULL;
		gsize len;

		if(g_file_get_contents(filename, &contents, &len, NULL)) {
			GtkTextMark *mark = NULL;
			GtkTextIter iter;

			mark = gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(demo->buffer_html));
			gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(demo->buffer_html), &iter, mark);

			talkatu_html_buffer_insert_html(TALKATU_HTML_BUFFER(demo->buffer_html), &iter, contents, len);
			g_free(contents);
		}

		g_free(filename);
	}

	gtk_widget_destroy(dialog);
}

static void
talkatu_demo_insert_markdown_cb(GtkButton *toggle, gpointer data) {
	TalkatuDemo *demo = (TalkatuDemo *)data;
	GtkWidget *dialog = NULL;
	GtkFileFilter *filter = NULL;
	gint res;

	dialog = gtk_file_chooser_dialog_new(
		_("insert markdown..."),
		GTK_WINDOW(demo),
		GTK_FILE_CHOOSER_ACTION_OPEN,
		"OK", GTK_RESPONSE_ACCEPT,
		"Cancel", GTK_RESPONSE_REJECT,
		NULL
	);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.md");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);

	res = gtk_dialog_run(GTK_DIALOG(dialog));
	if(res == GTK_RESPONSE_ACCEPT) {
		gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gchar *contents = NULL;
		gsize len;

		if(g_file_get_contents(filename, &contents, &len, NULL)) {
			GtkTextMark *mark = NULL;
			GtkTextIter iter;

			mark = gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(demo->buffer_markdown));
			gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(demo->buffer_markdown), &iter, mark);

			talkatu_markdown_buffer_insert_markdown(TALKATU_MARKDOWN_BUFFER(demo->buffer_markdown), &iter, contents, len);
			g_free(contents);
		}

		g_free(filename);
	}

	gtk_widget_destroy(dialog);
}

static void
talkatu_demo_buffer_changed_cb(GtkToggleButton *toggle, gpointer data) {
	TalkatuDemo *demo = (TalkatuDemo *)data;
	GtkWidget *view = talkatu_editor_get_view(TALKATU_EDITOR(demo->editor));

	if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(demo->toggle_plain))) {
		g_message("switching to plain buffer");
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), demo->buffer_plain);
	} else if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(demo->toggle_whole))) {
		g_message("switching to whole buffer");
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), demo->buffer_whole);
	} else if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(demo->toggle_html))) {
		g_message("switching to html buffer");
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), demo->buffer_html);
	} else if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(demo->toggle_markdown))) {
		g_message("switch to markdown buffer");
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), demo->buffer_markdown);
	}
}

static void
talkatu_demo_view_open_url_cb(TalkatuView *view, const gchar *url, gpointer data) {
	GError *error = NULL;
	gboolean success = FALSE;

#if GTK_CHECK_VERSION(3, 22, 0)
	success = gtk_show_uri_on_window(
		GTK_WINDOW(data),
		url,
		GDK_CURRENT_TIME,
		&error
	);
#else
	success = gtk_show_uri(
		gtk_widget_get_screen(view),
		url,
		GDK_CURRENT_TIME,
		&error
	);
#endif

	if(!success) {
		g_message(
			"failed to open uri '%s': %s",
			url,
			(error) ? error->message : NULL
		);

		g_error_free(error);
	}
}

static void
talkatu_demo_view_send_message_cb(TalkatuView *view, gpointer data) {
	TalkatuDemo *demo = TALKATU_DEMO(data);
	TalkatuContentType content_type = TALKATU_CONTENT_TYPE_PLAIN;
	TalkatuMessage *message = NULL;
	GtkTextBuffer *input = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	static guint64 id = 0;

	if(TALKATU_IS_HTML_BUFFER(input)) {
		content_type = TALKATU_CONTENT_TYPE_HTML;
	} else if(TALKATU_IS_MARKDOWN_BUFFER(input)) {
		content_type = TALKATU_CONTENT_TYPE_MARKDOWN;
	}

	message = talkatu_demo_message_new(
		id++,
		content_type,
		"demo1",
		talkatu_buffer_get_plain_text(TALKATU_BUFFER(input))
	);

	talkatu_history_buffer_write_message(
		TALKATU_HISTORY_BUFFER(demo->buffer_history),
		message
	);

	talkatu_buffer_clear(TALKATU_BUFFER(input));
	g_object_unref(G_OBJECT(message));
}

static gboolean
talkatu_demo_closed_cb(GtkWidget *w, GdkEvent *e, gpointer d) {
	gtk_main_quit();

	return FALSE;
}

static void
talkatu_demo_init(TalkatuDemo *demo) {
	gtk_widget_init_template(GTK_WIDGET(demo));

	gtk_text_view_set_buffer(GTK_TEXT_VIEW(demo->view_history), demo->buffer_history);
}

static void
talkatu_demo_class_init(TalkatuDemoClass *klass) {
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);

	gtk_widget_class_set_template_from_resource(
		widget_class,
		"/org/bitbucket/rw_grim/talkatu/ui/demo/demo.ui"
	);

	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, view_history);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, editor);

	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, buffer_plain);
 	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, buffer_whole);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, buffer_html);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, buffer_markdown);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, buffer_history);

	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, toggle_plain);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, toggle_whole);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, toggle_html);
	gtk_widget_class_bind_template_child(widget_class, TalkatuDemo, toggle_markdown);

	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_closed_cb);
	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_buffer_changed_cb);
	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_view_open_url_cb);
	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_view_send_message_cb);

	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_insert_html_cb);
	gtk_widget_class_bind_template_callback(widget_class, talkatu_demo_insert_markdown_cb);
}

static gboolean
talkatu_demo_version_cb(const gchar *n, const gchar *v, gpointer d, GError **e) {
	printf("talkatu-demo %s\n", TALKATU_VERSION);

	exit_early = TRUE;

	return TRUE;
}

static GOptionEntry entries[] = {
	{
		"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
		talkatu_demo_version_cb, N_("Display the version and exit"),
		NULL,
	}, {
		NULL, 0, 0, 0, NULL, NULL, NULL,
	}
};

gint
main(gint argc, gchar **argv) {
	GtkWidget *win = NULL;
	GError *error = NULL;
	GOptionContext *ctx = NULL;
	GOptionGroup *group = NULL;

	ctx = g_option_context_new("");
	g_option_context_set_summary(ctx, _("Talkatu Demo"));
	g_option_context_set_translation_domain(ctx, GETTEXT_PACKAGE);
	g_option_context_add_main_entries(ctx, entries, NULL);

	group = gtk_get_option_group(FALSE);
	g_option_context_add_group(ctx, group);

	g_option_context_parse(ctx, &argc, &argv, &error);
	g_option_context_free(ctx);

	if(error) {
		fprintf(stderr, "%s\n", error->message);

		g_error_free(error);

		talkatu_uninit();

		return EXIT_FAILURE;
	}

	if(exit_early) {
		talkatu_uninit();

		return EXIT_SUCCESS;
	}

	gtk_init(&argc, &argv);

	talkatu_init();

	win = g_object_new(talkatu_demo_get_type(), NULL);
	gtk_widget_show(win);

	gtk_main();

	talkatu_uninit();

	return 0;
}
