/*
 * talkatu
 * Copyright (C) 2017-2018 Gary Kramlich <grim@reaperworld.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
#include "talkatudemomessage.h"

typedef struct {
	guint64 id;
	TalkatuContentType content_type;
	gchar *author;
	gchar *contents;
} TalkatuDemoMessagePrivate;

enum {
	PROP_0,
	N_PROPERTIES,
	/* overrides */
	PROP_ID = N_PROPERTIES,
	PROP_CONTENT_TYPE,
	PROP_AUTHOR,
	PROP_CONTENTS,
};

static void
talkatu_demo_message_iface_init(TalkatuMessageInterface *iface) {
}

G_DEFINE_TYPE_EXTENDED(
	TalkatuDemoMessage,
	talkatu_demo_message,
	G_TYPE_OBJECT,
	0,
	G_ADD_PRIVATE(TalkatuDemoMessage)
	G_IMPLEMENT_INTERFACE(TALKATU_TYPE_MESSAGE, talkatu_demo_message_iface_init)
);

/******************************************************************************
 * Accessors
 *****************************************************************************/
static guint64
talkatu_demo_message_get_id(TalkatuDemoMessage *message) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	return priv->id;
}

static void
talkatu_demo_message_set_id(TalkatuDemoMessage *message, guint64 id) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	priv->id = id;

	g_object_notify(G_OBJECT(message), "id");
}

static TalkatuContentType
talkatu_demo_message_get_content_type(TalkatuDemoMessage *message) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	return priv->content_type;
}

static void
talkatu_demo_message_set_content_type(TalkatuDemoMessage *message, TalkatuContentType content_type) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	priv->content_type = content_type;

	g_object_notify(G_OBJECT(message), "content-type");
}

static gchar *
talkatu_demo_message_get_author(TalkatuDemoMessage *message) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	return priv->author;
}

static void
talkatu_demo_message_set_author(TalkatuDemoMessage *message, const gchar *author) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	g_free(priv->author);

	priv->author = author ? g_strdup(author) : NULL;

	g_object_notify(G_OBJECT(message), "author");
}

static gchar *
talkatu_demo_message_get_contents(TalkatuDemoMessage *message) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	return priv->contents;
}

static void
talkatu_demo_message_set_contents(TalkatuDemoMessage *message, const gchar *contents) {
	TalkatuDemoMessagePrivate *priv = talkatu_demo_message_get_instance_private(message);

	g_free(priv->contents);

	priv->contents = contents ? g_strdup(contents) : NULL;

	g_object_notify(G_OBJECT(message), "contents");
}

/******************************************************************************
 * GObject Stuff
 *****************************************************************************/
static void
talkatu_demo_message_get_property(GObject *obj,
                                  guint prop_id,
                                  GValue *value,
                                  GParamSpec *pspec)
{
	TalkatuDemoMessage *message = TALKATU_DEMO_MESSAGE(obj);

	switch(prop_id) {
		case PROP_ID:
			g_value_set_uint64(value, talkatu_demo_message_get_id(message));
			break;
		case PROP_CONTENT_TYPE:
			g_value_set_enum(value, talkatu_demo_message_get_content_type(message));
			break;
		case PROP_AUTHOR:
			g_value_set_string(value, talkatu_demo_message_get_author(message));
			break;
		case PROP_CONTENTS:
			g_value_set_string(value, talkatu_demo_message_get_contents(message));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
			break;
	}
}

static void
talkatu_demo_message_set_property(GObject *obj,
                                  guint prop_id,
                                  const GValue *value,
                                  GParamSpec *pspec)
{
	TalkatuDemoMessage *message = TALKATU_DEMO_MESSAGE(obj);

	switch(prop_id) {
		case PROP_ID:
			talkatu_demo_message_set_id(message, g_value_get_uint64(value));
			break;
		case PROP_CONTENT_TYPE:
			talkatu_demo_message_set_content_type(message, g_value_get_enum(value));
			break;
		case PROP_AUTHOR:
			talkatu_demo_message_set_author(message, g_value_get_string(value));
			break;
		case PROP_CONTENTS:
			talkatu_demo_message_set_contents(message, g_value_get_string(value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
			break;
	}
}

static void
talkatu_demo_message_finalize(GObject *obj) {
	TalkatuDemoMessage *message = TALKATU_DEMO_MESSAGE(obj);

	talkatu_demo_message_set_author(message, NULL);
	talkatu_demo_message_set_contents(message, NULL);

	G_OBJECT_CLASS(talkatu_demo_message_parent_class)->finalize(obj);
}

static void
talkatu_demo_message_init(TalkatuDemoMessage *toolbar) {
}

static void
talkatu_demo_message_class_init(TalkatuDemoMessageClass *klass) {
	GObjectClass *obj_class = G_OBJECT_CLASS(klass);

	obj_class->get_property = talkatu_demo_message_get_property;
	obj_class->set_property = talkatu_demo_message_set_property;
	obj_class->finalize = talkatu_demo_message_finalize;

	g_object_class_override_property(obj_class, PROP_ID, "id");
	g_object_class_override_property(obj_class, PROP_CONTENT_TYPE, "content-type");
	g_object_class_override_property(obj_class, PROP_AUTHOR, "author");
	g_object_class_override_property(obj_class, PROP_CONTENTS, "contents");
}

/******************************************************************************
 * Public API
 *****************************************************************************/
TalkatuMessage *talkatu_demo_message_new(guint64 id,
                                         TalkatuContentType content_type,
                                         const gchar *author,
                                         const gchar *contents)
{
	return TALKATU_MESSAGE(g_object_new(
		TALKATU_TYPE_DEMO_MESSAGE,
		"id", id,
		"content-type", content_type,
		"author", author,
		"contents", contents,
		NULL
	));
}
