/*
 * talkatu
 * Copyright (C) 2017-2018 Gary Kramlich <grim@reaperworld.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
#include <gtk/gtk.h>

#include <talkatu/talkatuhistory.h>
#include <talkatu/talkatumessage.h>
#include <talkatu/talkatumessageactions.h>

/**
 * SECTION:talkatuhistory
 * @Title: History View
 * @Short_description: Chat history widget
 *
 * This widget is used to display the conversation history.  It doesn't care if
 * it is multi user or one to one.
 */

G_DEFINE_TYPE(TalkatuHistory, talkatu_history, TALKATU_TYPE_VIEW)


/******************************************************************************
 * Signal Handlers
 *****************************************************************************/
static void
talkatu_history_buffer_insert_anchor_cb(GtkTextBuffer *textbuffer,
                                        GtkTextIter *location,
                                        GtkTextChildAnchor *anchor,
                                        gpointer user_data)
{
	gpointer data = NULL;

	data = g_object_get_data(G_OBJECT(anchor), "talkatu-message");
	if(data != NULL) {
		GtkTextView *text_view = GTK_TEXT_VIEW(user_data);
		GtkWidget *message_actions = talkatu_message_actions_new(TALKATU_MESSAGE(data));

		gtk_text_view_add_child_at_anchor(text_view, message_actions, anchor);

		gtk_widget_show_all(message_actions);
	}
}

static void
talkatu_history_buffer_set_cb(GObject *view, GParamSpec *pspec, gpointer data) {
	g_signal_connect_after(
		gtk_text_view_get_buffer(GTK_TEXT_VIEW(view)),
		"insert-child-anchor",
		G_CALLBACK(talkatu_history_buffer_insert_anchor_cb),
		view
	);
}

/******************************************************************************
 * GObject Stuff
 *****************************************************************************/
static void
talkatu_history_init(TalkatuHistory *history) {
	gtk_text_view_set_editable(GTK_TEXT_VIEW(history), FALSE);

	g_signal_connect_after(
		G_OBJECT(history),
		"notify::buffer",
		G_CALLBACK(talkatu_history_buffer_set_cb),
		NULL
	);
}

static void
talkatu_history_class_init(TalkatuHistoryClass *klass) {
}

/******************************************************************************
 * Public API
 *****************************************************************************/

/**
 * talkatu_history_new:
 *
 * Creates a new #TalkatuHistory for displaying chat history.
 *
 * Returns: (transfer full): The new #TalkatuHistory instance.
 */
GtkWidget *talkatu_history_new(void) {
	return GTK_WIDGET(g_object_new(
		TALKATU_TYPE_HISTORY,
		NULL
	));
}
